<?php

declare(strict_types=1);

namespace Drupal\Tests\prevent_homepage_deletion\Functional;

use Drupal\Tests\node\Functional\Views\NodeTestBase;

/**
 * Tests a node bulk form.
 *
 * @group prevent_homepage_deletion
 * @see \Drupal\node\Plugin\views\field\BulkForm
 */
class DeleteHomepageBulkActionTest extends NodeTestBase {

  /**
   * Modules to be enabled.
   *
   * @var array
   */
  protected static $modules = ['node_test_views', 'language', 'user', 'prevent_homepage_deletion'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Views used by this test.
   *
   * @var array
   */
  public static $testViews = ['test_node_bulk_form'];

  /**
   * Our node type.
   *
   * @var \Drupal\node\Entity\NodeType
   */
  protected $contentType;

  /**
   * Homepage node.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $pageHome;

  /**
   * {@inheritdoc}
   */
  protected function setUp($import_test_views = TRUE, $modules = ['node_test_views']): void {
    parent::setUp($import_test_views, $modules);

    $this->contentType = $this->createContentType(['type' => 'page']);
    // Create a first page (homepage).
    $this->pageHome = $this->drupalCreateNode(['type' => 'page']);
    // Set page to be homepage.
    \Drupal::configFactory()
      ->getEditable('system.site')
      ->set('page.front', '/node/' . $this->pageHome->id())
      ->save(TRUE);
  }

  /**
   * Test all bulk actions are available.
   */
  public function testBulkOperationsAvailable(): void {
    $this->drupalLogin($this->drupalCreateUser([
      'administer nodes',
      'access content overview',
    ]));
    $this->drupalGet('test-node-bulk-form');
    $elements = $this->assertSession()->selectExists('edit-action')->findAll('css', 'option');
    $this->assertCount(9, $elements, 'Not all node operations are found.');
  }

  /**
   * Tests the node bulk form.
   */
  public function testBulkForm(): void {
    // Step 1: Log in a user who cannot unpublish the homepage.
    $this->drupalLogin(
      $this->createUser([
        'delete any page content',
        'administer nodes',
        'access content overview',
        'delete any page content',
        'edit any page content',
        'create page content',
      ])
    );

    // Step 2: Try to unpublish the homepage.
    $this->assertTrue($this->pageHome->isPublished(), 'Homepage is initially published');
    $edit = [
      'node_bulk_form[0]' => TRUE,
      'action' => 'node_unpublish_action',
    ];
    $page = $this->drupalGet('admin/content');
    $this->submitForm($edit, 'Apply to selected items');
    $node = $this->loadNode($this->pageHome->id());
    $this->assertTrue($node->isPublished(), 'Homepage has been unpublished, it should be published');

    // Step 3: Logout, and login as user with the permission.
    $this->drupalLogout();
    $this->drupalLogin(
      $this->createUser([
        'delete_homepage_node',
        'administer nodes',
        'access content overview',
        'delete any page content',
        'edit any page content',
        'create page content',
      ])
    );

    // Step 4: Try to unpublish the homepage.
    $this->assertTrue($this->pageHome->isPublished(), 'Homepage is initially published');
    $edit = [
      'node_bulk_form[0]' => TRUE,
      'action' => 'node_unpublish_action',
    ];
    $page = $this->drupalGet('admin/content');
    $this->submitForm($edit, 'Apply to selected items');
    $node = $this->loadNode($this->pageHome->id());
    $this->assertFalse($node->isPublished(), 'Node has not been unpublished');
  }

  /**
   * Tests multiple deletion.
   */
  public function testBulkDeletion(): void {
    $label = $this->loadNode($this->pageHome->id())->label();

    // Step 1: Log in a user who cannot delete the homepage.
    $this->drupalLogin(
      $this->createUser([
        'delete any page content',
        'administer nodes',
        'access content overview',
        'delete any page content',
        'edit any page content',
        'create page content',
      ])
    );

    // Step 2: Try to delete the homepage.
    $node = $this->loadNode($this->pageHome->id());
    $this->assertNotNull($node, 'Home has been deleted');
    $page = $this->drupalGet('admin/content');
    $edit = [
      'node_bulk_form[0]' => TRUE,
      'action' => 'node_delete_action',
    ];
    $this->submitForm($edit, 'Apply to selected items');
    $this->assertSession()->pageTextContains("You tried to delete the homepage, but you do not have the permission to do so.");
    $node = $this->loadNode($this->pageHome->id());
    $this->assertNotNull($node, 'Homepage has been deleted, though it should not be possible');

    // Step 3: Logout, and login as user with the permission.
    $this->drupalLogout();
    $this->drupalLogin(
      $this->createUser([
        'delete_homepage_node',
        'administer nodes',
        'access content overview',
        'delete any page content',
        'edit any page content',
        'create page content',
      ])
    );

    // Step 4: Try to delete the homepage.
    $node = $this->loadNode($this->pageHome->id());
    $this->assertNotNull($node, 'Home has unintentionally been deleted');
    $page = $this->drupalGet('admin/content');
    $edit = [
      'node_bulk_form[0]' => TRUE,
      'action' => 'node_delete_action',
    ];
    $this->submitForm($edit, 'Apply to selected items');
    $this->assertSession()->pageTextContains($label);
    $this->submitForm([], 'Delete');
    $node = $this->loadNode($this->pageHome->id());
    $this->assertNull($node, 'Homepage has not been deleted, though it should have been');
  }

  /**
   * Load the specified node from the storage.
   *
   * @param int $id
   *   The node identifier.
   *
   * @return \Drupal\node\NodeInterface
   *   The loaded node.
   */
  protected function loadNode($id) {
    /** @var \Drupal\node\NodeStorage $storage */
    $storage = $this->container->get('entity_type.manager')->getStorage('node');
    $storage->resetCache([$id]);
    return $storage->load($id);
  }

}
